unit DGLHostIntf;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, DGLIntf, dynlibs, contnrs, fgl;

type
  EBenchmarkLibException = class (Exception);

  TBenchmarkResultMapValue = class (TObject)
  public
    function ToString: String; virtual; abstract;
  end;
  TBenchmarkResultMapValues = specialize TFPGList<TBenchmarkResultMapValue>;

  { TBenchmarkResultMapValueStr }

  TBenchmarkResultMapValueStr = class (TBenchmarkResultMapValue)
  public
    constructor Create(const AValue: String);
  private
    FValue: String;
  public
    function ToString: String; override;
  end;

  { TBenchmarkResultMapValueFloat }

  TBenchmarkResultMapValueFloat = class (TBenchmarkResultMapValue)
  public
    constructor Create(const AValue: Double);
  private
    FValue: Double;
  public
    function ToString: String; override;
  end;

  { TBenchmarkResultMapValueInt }

  TBenchmarkResultMapValueInt = class (TBenchmarkResultMapValue)
  public
    constructor Create(const AValue: LongInt);
  private
    FValue: LongInt;
  public
    function ToString: String; override;
  end;

  { TBenchmarkResultMap }

  TBenchmarkResultMap = class (TInterfacedObject, IBenchmarkResultMap)
  public
    constructor Create(Fixed: Boolean = False);
    destructor Destroy; override;
  private
    FFixed: Boolean;
    FMap: TFPObjectHashTable;
    FPerfValues: TBenchmarkResultMapValues;
    FResultList: TStringList;
    function GetResult(Index: Integer): TBenchmarkResultMapValue;
    function GetResultByKeyAsString(AKey: String): String;
    function GetResultCount: Integer;
  protected
    function GetResultByKey(const Key: String): TBenchmarkResultMapValue;
    procedure SetResultValue(const AKey: String; const AValue: TBenchmarkResultMapValue);
    procedure DeleteResult(const AKey: String);
  public
    property Result[Index: Integer]: TBenchmarkResultMapValue read GetResult;
    property ResultByKey[AKey: String]: TBenchmarkResultMapValue read GetResultByKey;
    property ResultByKeyAsString[AKey: String]: String read GetResultByKeyAsString;
    property ResultCount: Integer read GetResultCount;
  public
    procedure BeforeDestruction; override;
  public
    {$I DGLIntfCalling.inc}
    function GetDefaultResultName(const Key: TBenchmarkResultKey): PChar;
    procedure SetResult(const Key: PChar; const Value: PChar; Len: LongInt);
    procedure SetResultInt(const Key: PChar; const Value: LongInt);
    procedure SetResultFloat(const Key: PChar; const Value: Double);
    procedure UnsetResult(const Key: PChar);
    {$CALLING register}
  end;

  { TBenchmarkTimer }

  TBenchmarkTimer = class (TInterfacedObject, IBenchmarkTimer)
  public
    constructor Create; virtual;
  private
    FTime: Double;
  protected
    function DoGetTime: Double; virtual; abstract;
  public
    {$I DGLIntfCalling.inc}
    procedure Reset;
    function GetInterval: Double;
    {$CALLING register}
  end;
  TBenchmarkTimerClass = class of TBenchmarkTimer;

  { TBenchmarkLibHandle }

  TBenchmarkLibHandle = class (TObject, IBenchmarkLib)
  public
    constructor Create(const ALibName: String); virtual;
    destructor Destroy; override;
  private
    FLibHandle: TLibHandle;
    FLibInterface: IBenchmarkLib;
  published
    property LibInterface: IBenchmarkLib read FLibInterface implements IBenchmarkLib;
  end;

var
  BenchmarkTimerClass: TBenchmarkTimerClass = nil;

implementation

{ TBenchmarkResultMapValueStr }

constructor TBenchmarkResultMapValueStr.Create(const AValue: String);
begin
  FValue := AValue;
end;

function TBenchmarkResultMapValueStr.ToString: String;
begin
  Result := FValue;
end;

{ TBenchmarkResultMapValueFloat }

constructor TBenchmarkResultMapValueFloat.Create(const AValue: Double);
begin
  FValue := AValue;
end;

function TBenchmarkResultMapValueFloat.ToString: String;
begin
  Result := Format('%.4f', [FValue]);
end;

{ TBenchmarkResultMapValueInt }

constructor TBenchmarkResultMapValueInt.Create(const AValue: LongInt);
begin
  FValue := AValue;
end;

function TBenchmarkResultMapValueInt.ToString: String;
begin
  Result := IntToStr(FValue);
end;

{ TBenchmarkResultMap }

constructor TBenchmarkResultMap.Create(Fixed: Boolean);
begin
  FMap := TFPObjectHashTable.Create(True);
  FPerfValues := TBenchmarkResultMapValues.Create;
  FResultList := TStringList.Create;
  FFixed := Fixed;
  if FFixed then
    _AddRef;
end;

destructor TBenchmarkResultMap.Destroy;
begin
  FPerfValues.Free;
  FResultList.Free;
  FMap.Free;
  inherited Destroy;
end;

function TBenchmarkResultMap.GetResult(Index: Integer
  ): TBenchmarkResultMapValue;
begin
  Result := TBenchmarkResultMapValue(FResultList[Index]);
end;

function TBenchmarkResultMap.GetResultByKeyAsString(AKey: String): String;
var
  Value: TBenchmarkResultMapValue;
begin
  Value := GetResultByKey(AKey);
  if Value = nil then
    Result := ''
  else
    Result := Value.ToString;
end;

function TBenchmarkResultMap.GetResultCount: Integer;
begin
  Result := FResultList.Count;
end;

function TBenchmarkResultMap.GetResultByKey(const Key: String
  ): TBenchmarkResultMapValue;
begin
  Result := TBenchmarkResultMapValue(FMap.Items[Key]);
end;

procedure TBenchmarkResultMap.SetResultValue(const AKey: String;
  const AValue: TBenchmarkResultMapValue);
var
  Existing: TObject;
begin
  Existing := FMap.Items[AKey];
  if Existing <> nil then
    Existing.Free;
  FMap.Items[AKey] := AValue;
end;

procedure TBenchmarkResultMap.DeleteResult(const AKey: String);
var
  Existing: TObject;
begin
  Existing := FMap.Items[AKey];
  if Existing <> nil then
  begin
    Existing.Free;
    FMap.Delete(AKey);
  end;
end;

procedure TBenchmarkResultMap.BeforeDestruction;
begin
  if not FFixed then
    inherited BeforeDestruction;
end;

{$I DGLIntfCalling.inc}

function TBenchmarkResultMap.GetDefaultResultName(const Key: TBenchmarkResultKey
  ): PChar;
begin
  case Key of
    BRK_FPS: Result := BRKN_FPS;
    BRK_TIME: Result := BRKN_TIME;
    BRK_MESSAGE: Result := BRKN_MESSAGE;
    BRK_IMAGE: Result := BRKN_IMAGE;
    BRK_IMAGE_MIME: Result := BRKN_IMAGE_MIME;
  end;
end;

procedure TBenchmarkResultMap.SetResult(const Key: PChar; const Value: PChar;
  Len: LongInt);
var
  KeyName: String;
  Data: String;
  Existing: TBenchmarkResultMapValue;
begin
  KeyName := StrPas(Key);
  SetLength(Data, Len);
  Move(Data[1], Value^, Len);

  if Pos(BRKC_PERFORMANCE+'/', KeyName) = 1 then
    Exit; // Do not allow string values in perf keys.
  SetResultValue(KeyName, TBenchmarkResultMapValueStr.Create(Data));
end;

procedure TBenchmarkResultMap.SetResultInt(const Key: PChar;
  const Value: LongInt);
var
  KeyName: String;
  Existing: TBenchmarkResultMapValue;
begin
  KeyName := StrPas(Key);

  if (Pos(BRKC_MIME+'/', KeyName) = 1) or (Pos(BRKC_DATA+'/', KeyName) = 1) then
    Exit; // Do not allow int or float in data/ or mime/.

  if Pos(BRKC_PERFORMANCE+'/', KeyName) = 1 then
  begin
    // Performance keys should be floats, so convert it to float.
    SetResultValue(KeyName, TBenchmarkResultMapValueFloat.Create(Value));
  end
  else
    SetResultValue(KeyName, TBenchmarkResultMapValueInt.Create(Value));
end;

procedure TBenchmarkResultMap.SetResultFloat(const Key: PChar;
  const Value: Double);
var
  KeyName: String;
  Existing: TBenchmarkResultMapValue;
begin
  KeyName := StrPas(Key);


  if (Pos(BRKC_MIME+'/', KeyName) = 1) or (Pos(BRKC_DATA+'/', KeyName) = 1) then
    Exit; // Do not allow int or float in data/ or mime/.

  SetResultValue(KeyName, TBenchmarkResultMapValueFloat.Create(Value));
end;

procedure TBenchmarkResultMap.UnsetResult(const Key: PChar);
var
  KeyName: String;
  Existing: TBenchmarkResultMapValue;
begin
  KeyName := StrPas(Key);
  DeleteResult(KeyName);
end;
{$CALLING register}

{ TBenchmarkTimer }

constructor TBenchmarkTimer.Create;
begin

end;

{$I DGLIntfCalling.inc}

procedure TBenchmarkTimer.Reset;
begin
  FTime := DoGetTime;
end;

function TBenchmarkTimer.GetInterval: Double;
var
  FNewTime: Double;
begin
  FNewTime := DoGetTime;
  Result := FNewTime - FTime;
  FTime := FNewTime;
end;

{$CALLING register}

{ TBenchmarkLibHandle }

constructor TBenchmarkLibHandle.Create(const ALibName: String);
var
  InitCall: TBenchmarkLibHello;
begin
  FLibHandle := LoadLibrary(ALibName);
  if FLibHandle = 0 then
    raise EBenchmarkLibException.CreateFmt('Could not load library ''%s''.', [ALibName]);
  try
    InitCall := TBenchmarkLibHello(GetProcAddress(FLibHandle, 'Hello'));
    if InitCall = nil then
      raise EBenchmarkLibException.CreateFmt('Library ''%s'' does not expose ''Hello'' function.', [ALibName]);
    InitCall(FLibInterface);
    if (FLibInterface = nil) then
      raise EBenchmarkLibException.CreateFmt('Library ''%s'' did not supply interface.', [ALibName]);
  except
    FreeLibrary(FLibHandle);
  end;
end;

destructor TBenchmarkLibHandle.Destroy;
begin
  FLibInterface := nil;
  FreeLibrary(FLibHandle);
  inherited Destroy;
end;

end.

