unit Main; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  StdCtrls, DGLIntf, DGLHostIntf;

const
  {$IFDEF WINDOWS}
  LIB_EXTENSION = '.dll';
  {$ELSE}
  LIB_EXTENSION = '.so';
  {$ENDIF}

type

  { TMainForm }

  TMainForm = class(TForm, IBenchmarkHost)
    Guests: TListBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure GuestsDblClick(Sender: TObject);
  private
    FBenchmarks: TInterfaceList;
    FLibs: TFPList;
    FTimer: IBenchmarkTimer;
  public
    procedure PerformBenchmark(AGuest: IBenchmarkGuest);
    procedure RegisterGuest(AGuest: IBenchmarkGuest; AName, ADescription: String);
    procedure ScanBenchmarks;
  public
    {$I intf/DGLIntfCalling.inc}
    procedure Add(Guest: IBenchmarkGuest);
    function GetTimer: IBenchmarkTimer;
    {$CALLING register}
  end; 

var
  MainForm: TMainForm;

implementation

{ TMainForm }

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FTimer := BenchmarkTimerClass.Create;
  FLibs := TFPList.Create;
  FBenchmarks := TInterfaceList.Create;
  ScanBenchmarks;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  FBenchmarks.Free;
  FLibs.Free;
end;

procedure TMainForm.GuestsDblClick(Sender: TObject);
var
  Idx: Integer;
begin
  Idx := Guests.ItemIndex;
  if Idx < 0 then
    Exit;
  PerformBenchmark(IBenchmarkGuest(Pointer(Guests.Items.Objects[Idx])));
end;

procedure TMainForm.PerformBenchmark(AGuest: IBenchmarkGuest);
var
  Map: TBenchmarkResultMap;
  Needs: TBenchmarkNeeds;
  Run: Integer;
const
  RUN_COUNT = 10;
begin
  Map := TBenchmarkResultMap.Create(True);
  try
    Needs := AGuest.GetNeeds;
    case AGuest.Prepare(Self) of
      brUnknownError:
      begin
        ShowMessage('Benchmark raised an unknown error.');
        Exit;
      end;
      brInsufficientHardware:
      begin
        ShowMessage('Your hardware does not match the conditions to run this benchmark.');
        Exit;
      end;
      brAbortedByUser:
      begin
        ShowMessage('The benchmark was aborted by the user.');
        Exit;
      end;
      brInsufficientHostEnvironment:
      begin
        ShowMessage('The benchmark is not pleased with this host environment.');
        Exit;
      end;
      brOkay: ;
    else
      ShowMessage('Benchmark raised unknown error.');
      Exit;
    end;
    case AGuest.RunBenchmark(Map) of
      brUnknownError: ShowMessage('Benchmark raised an unknown error.');
      brInsufficientHardware: ShowMessage('Your hardware does not match the conditions to run this benchmark.');
      brAbortedByUser: ShowMessage('The benchmark was aborted by the user.');
      brOkay: ShowMessage(Format('%s: %s%s%s: %s', [BRKN_TIME, Map.ResultByKeyAsString[BRKN_TIME], LineEnding, BRKN_FPS, Map.ResultByKeyAsString[BRKN_FPS]]));
    else
      ShowMessage('Benchmark raised unknown error.');
    end;
    AGuest.Finish;

  finally
    Map.Free;
  end;
end;

procedure TMainForm.RegisterGuest(AGuest: IBenchmarkGuest; AName,
  ADescription: String);
begin
  FBenchmarks.Add(AGuest);
  Guests.Items.AddObject(AName, TObject(Pointer(AGuest)));
end;

procedure TMainForm.ScanBenchmarks;
var
  SearchRec: TSearchRec;
  Root: String;
  Lib: TBenchmarkLibHandle;
begin
  Root := ExtractFilePath(Application.ExeName) + 'benchmarks' + PathDelim ;
  try
    if FindFirst(Root+ '*' + LIB_EXTENSION, faAnyFile, SearchRec) <> 0 then
      Exit;
    repeat
      if (SearchRec.Name = '.') or (SearchRec.Name = '..') then
        Continue;
      try
        Lib := TBenchmarkLibHandle.Create(Root + SearchRec.Name);
        FLibs.Add(Lib);
        Lib.LibInterface.Hello(Self);
      except
        on E: EBenchmarkLibException do
        begin
          MessageDlg(E.Message, mtError, [mbOk], 0);
        end
        else
          raise;
      end;
    until FindNext(SearchRec) <> 0;
  finally
    FindClose(SearchRec);
  end;
end;

{$I intf/DGLIntfCalling.inc}

procedure TMainForm.Add(Guest: IBenchmarkGuest);
var
  PasIntf: IBenchmarkGuestPas;
  StrLen: LongInt;
  AName: String;
  ADescription: String;
begin
  if Guest.QueryInterface(IBenchmarkGuestPas, PasIntf) = S_OK then
  begin
    // We can use the more efficient and easier pascal interface.
    RegisterGuest(Guest, PasIntf.GetName, PasIntf.GetDescription);
  end
  else
  begin
    Guest.GetName(nil, StrLen);
    SetLength(AName, StrLen);
    Guest.GetName(@AName[1], StrLen);

    Guest.GetDescription(nil, StrLen);
    SetLength(ADescription, StrLen);
    Guest.GetDescription(@ADescription[1], StrLen);

    RegisterGuest(Guest, AName, ADescription);
  end;
end;

function TMainForm.GetTimer: IBenchmarkTimer;
begin
  Result := FTimer
end;
{$CALLING register}

initialization
  {$I main.lrs}

end.

